import collections.abc
from operator import eq, ge, gt, le, lt, ne
from textwrap import dedent

from .sortedcontainer import SortedContainer


class SortedList(SortedContainer):
    """A sorted list with efficient query performance and memory usage.

    The list is initialised with the content of the provided iterable ``arg``.

    The ``typecode`` argument is a single character that restricts the
    type of the stored elements. Type codes are defined in the 
    `array <https://docs.python.org/3/library/array.html>`_ module of the
    standard library.  If no type code is specified, the type is inferred
    from the contents of ``arg``.

    The ``epsilon`` argument allows to trade off memory usage with query
    performance. The default value is adequate in most cases.

    Methods for adding and removing elements:

    * :func:`SortedList.__add__`
    * :func:`SortedList.__sub__`
    * :func:`SortedList.drop_duplicates`

    Methods for accessing and querying elements:

    * :func:`SortedList.__getitem__`
    * :func:`SortedList.__contains__`
    * :func:`SortedList.bisect_left`
    * :func:`SortedList.bisect_right`
    * :func:`SortedList.count`
    * :func:`SortedList.find_ge`
    * :func:`SortedList.find_gt`
    * :func:`SortedList.find_le`
    * :func:`SortedList.find_lt`
    * :func:`SortedList.index`
    * :func:`SortedList.rank`

    Methods for iterating elements:

    * :func:`SortedList.range`
    * :func:`SortedList.__iter__`
    * :func:`SortedList.__reversed__`

    Methods for lexicographical comparisons:

    * :func:`SortedList.__eq__`
    * :func:`SortedList.__ne__`
    * :func:`SortedList.__ge__`
    * :func:`SortedList.__gt__`
    * :func:`SortedList.__le__`
    * :func:`SortedList.__lt__`

    Other methods:

    * :func:`SortedList.copy`
    * :func:`SortedList.stats`
    * :func:`SortedList.__repr__`

    Args:
        arg (iterable, optional): initial elements. Defaults to None.
        typecode (char, optional): type of the stored elements. Defaults
            to None.
        epsilon (int, optional): space-time trade-off parameter. Defaults
            to 64.

    Example:
        >>> from pygm import SortedList
        >>> sl = SortedList([0, 1, 34, 144, 1, 55, 233, 2, 3, 21, 89, 5, 8, 13])
        >>> sl
        SortedList([0, 1, 1, ..., 144, 233])
        >>> sl.find_gt(9)                                   # smallest element > 9
        13
        >>> sl.count(1)                                     # number of elements == 1
        2
        >>> 42 in sl                                        # membership test
        False
        >>> list(sl.range(0, 21, inclusive=(False, True)))  # elements 0 < x <= 21
        [1, 1, 2, 3, 5, 8, 13, 21]
        >>> sl[2:10:3]                                      # slicing syntax support
        SortedList([1, 5, 21])
        >>> (sl + [-3, -2, -1]).rank(0)                     # number of elements <= 0
        4
    """

    def __init__(self, arg=None, typecode=None, epsilon=64):
        SortedContainer._initwitharg(self, arg, typecode, epsilon, False)

    def __getitem__(self, i):
        """Return the element at position ``i``.

        ``self.__getitem__(i)`` <==> ``self[i]``

        Args:
            i (int or slice): index of the element

        Returns:
            element at position ``i``
        """
        if isinstance(i, slice):
            return SortedList(self._impl.slice(i), self._typecode)
        return self._impl[i]

    def __add__(self, other):
        """Return a new ``SortedList`` by merging the elements of ``self``
        with ``other``.

        ``self.__add__(other)`` <==> ``self + other``

        Values in ``other`` do not need to be in sorted order.

        Args:
            other (iterable): a sequence of values

        Returns:
            SortedList: new list with the merged elements
        """
        args = SortedContainer._impl_or_iter(other)
        return SortedList(self._impl.merge(*args), self._typecode)

    def __sub__(self, other):
        """Return a new ``SortedList`` by removing from ``self`` the elements
        found in ``other``.

        Equivalent elements are treated individually, that is, if some element
        is found m times in ``self`` and n times in ``other``, it will appear
        max(m-n, 0) times in the result.

        ``self.__sub__(other)`` <==> ``self - other``

        Values in ``other`` do not need to be in sorted order.

        Args:
            other (iterable): a sequence of values

        Returns:
            SortedList: new list with the elements in the difference
        """
        args = SortedContainer._impl_or_iter(other)
        return SortedList(self._impl.difference(*args), self._typecode)

    def drop_duplicates(self):
        """Return ``self`` with duplicate elements removed.

        Returns:
            SortedList: new list without duplicates
        """
        return SortedList(self._impl.drop_duplicates(), self._typecode)

    def copy(self):
        """Return a copy of ``self``.

        Returns:
            SortedList: new list with the same elements of ``self``
        """
        return SortedList(self._impl, self._typecode)

    __copy__ = copy

    def _make_cmp(op, symbol, doc):
        # credits: https://github.com/grantjenks/python-sortedcontainers/
        def comparer(self, other):
            if not isinstance(other, collections.abc.Sequence):
                return NotImplemented

            len_self = len(self)
            len_other = len(other)

            if len_self != len_other:
                if op is eq:
                    return False
                if op is ne:
                    return True

            for x, y in zip(self, other):
                if x != y:
                    return op(x, y)

            return op(len_self, len_other)

        op_name = op.__name__
        comparer.__name__ = '__{0}__'.format(op_name)
        doc_str = """Return ``True`` if and only if ``self`` is {0} ``other``.

        ``self.__{1}__(other)`` <==> ``self {2} other``

        Comparisons use the `lexicographical order <https://docs.python.org/3/tutorial/datastructures.html#comparing-sequences-and-other-types>`_.

        Args:
            other (iterable): a sequence of values

        Returns:
            bool: ``True`` if sorted list is {0} `other`
        """
        comparer.__doc__ = dedent(doc_str.format(doc, op_name, symbol))
        return comparer

    __eq__ = _make_cmp(eq, '==', 'equal to')
    __ne__ = _make_cmp(ne, '!=', 'not equal to')
    __lt__ = _make_cmp(lt, '<', 'less than')
    __gt__ = _make_cmp(gt, '>', 'greater than')
    __le__ = _make_cmp(le, '<=', 'less than or equal to')
    __ge__ = _make_cmp(ge, '>=', 'greater than or equal to')
    _make_cmp = staticmethod(_make_cmp)



import collections.abc

from . import _pygm


class SortedContainer(collections.abc.Sequence):
    @staticmethod
    def _fromtypecode(typecode, *args):
        if typecode in 'BHI':
            return _pygm.PGMIndexUInt32(*args)
        elif typecode in 'LQN':
            return _pygm.PGMIndexUInt64(*args)
        elif typecode in 'bhi':
            return _pygm.PGMIndexInt32(*args)
        elif typecode in 'lqn':
            return _pygm.PGMIndexInt64(*args)
        elif typecode in 'ef':
            return _pygm.PGMIndexFloat(*args)
        elif typecode in 'd':
            return _pygm.PGMIndexDouble(*args)
        else:
            raise TypeError('Unsupported typecode')

    @staticmethod
    def _impl_or_iter(o):
        n = len(o) if hasattr(o, '__len__') else 0
        o = o._impl if isinstance(o, SortedContainer) else iter(o)
        return (o, n)

    @staticmethod
    def _initwitharg(self, o, typecode, epsilon, drop_duplicates):
        has_len = hasattr(o, '__len__')
        if o is None or (has_len and len(o) == 0):
            self._typecode = 'q'
            self._impl = _pygm.PGMIndexInt64()
            return

        # Init from internal _pygm objects
        if isinstance(o, (_pygm.PGMIndexUInt32, _pygm.PGMIndexUInt64,
                          _pygm.PGMIndexInt32, _pygm.PGMIndexInt64,
                          _pygm.PGMIndexFloat, _pygm.PGMIndexDouble)):
            assert not (drop_duplicates and o.has_duplicates())
            self._typecode = typecode
            self._impl = o
            return

        # Init from an iterable
        is_iterable = isinstance(o, collections.abc.Iterable)
        if is_iterable:
            len_hint = len(o) if has_len else 0
            args = (len_hint, drop_duplicates, epsilon)
            tinit = SortedContainer._fromtypecode

            if typecode:  # user-provided typecode
                self._typecode = typecode
                self._impl = tinit(typecode, iter(o), *args)
                return

            try:  # try to get the typecode from memoryview
                v = memoryview(o)
                self._typecode = v.format
                self._impl = tinit(v.format, iter(v), *args)
                return
            except TypeError:
                pass

            # Find the typecode by inspecting the type of the elements
            anyfloat = any(isinstance(x, float) for x in o)
            self._typecode = 'd' if anyfloat else 'q'
            self._impl = tinit(self._typecode, iter(o), *args)
            return

        raise TypeError('Unsupported argument type')

    def __len__(self):
        """Return the number of elements in ``self``.

        ``self.__len__()`` <==> ``len(self)``

        Returns:
            int: number of elements
        """
        return self._impl.__len__()

    def __contains__(self, x):
        """Check whether ``self`` contains the given value ``x`` or not.

        ``self.__repr__(x)`` <==> ``x in self``

        Args:
            x: value to search

        Returns:
            bool: ``True`` if an element equal to ``x`` is found, ``False``
                otherwise
        """
        return self._impl.__contains__(x)

    def bisect_left(self, x):
        """Locate the insertion point for ``x`` to maintain sorted order.

        If ``x`` is already present, the insertion point will be before (to
        the left of) any existing entries.

        Similar to the `bisect <https://docs.python.org/3/library/bisect.html>`_
        module in the standard library.

        Args:
            x: value to compare the elements to

        Returns:
            int: insertion index in sorted list
        """
        return self._impl.bisect_left(x)

    def bisect_right(self, x):
        """Locate the insertion point for ``x`` to maintain sorted order.

        If ``x`` is already present, the insertion point will be after (to the
        right of) any existing entries.

        Similar to the `bisect <https://docs.python.org/3/library/bisect.html>`_
        module in the standard library.

        Args:
            x: value to compare the elements to

        Returns:
            int: insertion index in sorted list
        """
        return self._impl.bisect_right(x)

    def find_lt(self, x):
        """Find the rightmost element less than ``x``.

        Args:
            x: value to compare the elements to

        Returns:
            value of the rightmost element ``< x``, or ``None`` if no such
            element is found
        """
        return self._impl.find_lt(x)

    def find_le(self, x):
        """Find the rightmost element less than or equal to ``x``

        Args:
            x: value to compare the elements to

        Returns:
            value of the rightmost element ``<= x``, or ``None`` if no such
            element is found
        """
        return self._impl.find_le(x)

    def find_gt(self, x):
        """Find the leftmost element greater than ``x``.

        Args:
            x: value to compare the elements to

        Returns:
            value of the leftmost element ``> x``, or ``None`` if no such
            element is found
        """
        return self._impl.find_gt(x)

    def find_ge(self, x):
        """Find the leftmost element greater than or equal to ``x``.

        Args:
            x: value to compare the elements to

        Returns:
            value of the leftmost element ``>= x``, or ``None`` if no such
            element is found
        """
        return self._impl.find_ge(x)

    def rank(self, x):
        """Return the number of elements less than or equal to ``x``

        Args:
            x: value to compare the elements to

        Returns:
            int: number of elements ``<= x``
        """
        return self._impl.rank(x)

    def count(self, x):
        """Return the number of elements equal to ``x``.

        Args:
            x: value to count

        Returns:
            int: number of elements ``== x``
        """
        return self._impl.count(x)

    def range(self, a, b, inclusive=(True, True), reverse=False):
        """Return an iterator over elements between ``a`` and ``b``.

        Args:
            a: lower bound value
            b: upper bound value
            inclusive (tuple[bool, bool], optional): a pair of boolean
                indicating whether the bounds are inclusive (``True``) or
                exclusive (``False``). Defaults to ``(True, True)``.
            reverse (bool, optional): if ``True`` return an reverse iterator.
                Defaults to ``False``.

        Returns:
            iterator over the elements between the given bounds
        """
        return self._impl.range(a, b, inclusive, reverse)

    def index(self, x, start=None, stop=None):
        """Return the first index of ``x``.

        Args:
            x: element in the sorted list
            start (int, optional): restrict the search to the elements from
                this position onwards. Defaults to ``None``.
            stop (int, optional): restrict the search to the elements before
                this position. Defaults to ``None``.

        Returns:
            int: first index of ``x``

        Raises:
            ValueError: if ``x`` is not present
        """
        return self._impl.index(x, start, stop)

    def stats(self):
        """Return a dict containing statistics about ``self``.

        The keys are:

        * ``'data size'`` size of the elements in bytes
        * ``'index size'`` size of the index in bytes
        * ``'leaf segments'`` number of segments in the last level of the index
        * ``'height'`` number of levels of the index
        * ``'epsilon'`` value of the trade-off parameter of the index
        * ``'typecode'`` type of the elements

        Returns:
            dict[str, object]: a dictionary with stats about ``self``
        """
        d = self._impl.stats()
        d['typecode'] = self._typecode
        return d

    def __iter__(self):
        """Return an iterator over the elements of ``self``.

        ``self.__iter__()`` <==> ``iter(self)``

        Returns:
            iterator: iterator over the elements
        """
        return self._impl.__iter__()

    def __reversed__(self):
        """Return a reverse iterator over the elements of ``self``.

        ``self.__iter__()`` <==> ``iter(self)``

        Returns:
            iterator: reverse iterator over the elements
        """
        return self._impl.__reversed__()

    def __repr__(self):
        """Return a string representation of self.

        ``self.__repr__()`` <==> ``repr(self)``

        Returns:
            str: repr(self)
        """
        preview = ''
        if len(self) < 6:
            preview += repr(list(self._impl))
        else:
            fmt_args = (self[0], self[1], self[2], self[-2], self[-1])
            if self._typecode in 'fd':
                preview += '[%g, %g, %g, ..., %g, %g]' % fmt_args
            else:
                preview += '[%d, %d, %d, ..., %d, %d]' % fmt_args
        return '%s(%s)' % (self.__class__.__name__, preview)



from .sortedcontainer import SortedContainer


class SortedSet(SortedContainer):
    """A sorted set with efficient query performance and memory usage.

    The set is initialised with the content of the provided iterable ``arg``.

    The ``typecode`` argument is a single character that restricts the
    type of the stored elements. Type codes are defined in the 
    `array <https://docs.python.org/3/library/array.html>`_ module of the
    standard library.  If no type code is specified, the type is inferred
    from the contents of ``arg``.

    The ``epsilon`` argument allows to trade off memory usage with query
    performance. The default value is adequate in most cases.

    Methods for set operations:

    * :func:`SortedSet.difference` (alias for ``set - other``)
    * :func:`SortedSet.intersection` (alias for ``set & other``)
    * :func:`SortedSet.union` (alias for ``set | other``)
    * :func:`SortedSet.symmetric_difference` (alias for ``set ^ other``)

    Methods for accessing and querying elements:

    * :func:`SortedSet.__getitem__`
    * :func:`SortedSet.__contains__`
    * :func:`SortedSet.bisect_left`
    * :func:`SortedSet.bisect_right`
    * :func:`SortedSet.count`
    * :func:`SortedSet.find_ge`
    * :func:`SortedSet.find_gt`
    * :func:`SortedSet.find_le`
    * :func:`SortedSet.find_lt`
    * :func:`SortedSet.index`
    * :func:`SortedSet.rank`

    Methods for set comparisons:

    * :func:`SortedSet.__eq__` (set equality)
    * :func:`SortedSet.__ne__` (set inequality)
    * :func:`SortedSet.__ge__` (superset)
    * :func:`SortedSet.__gt__` (proper superset)
    * :func:`SortedSet.__le__` (subset)
    * :func:`SortedSet.__lt__` (proper subset)

    Methods for iterating elements:

    * :func:`SortedSet.range`
    * :func:`SortedSet.__iter__`
    * :func:`SortedSet.__reversed__`

    Other methods:

    * :func:`SortedSet.copy`
    * :func:`SortedSet.stats`
    * :func:`SortedSet.__repr__`

    Args:
        arg (iterable, optional): initial elements. Defaults to None.
        typecode (char, optional): type of the stored elements. Defaults
            to None.
        epsilon (int, optional): space-time trade-off parameter. Defaults
            to 64.
    """

    def __init__(self, arg=None, typecode=None, epsilon=64):
        SortedContainer._initwitharg(self, arg, typecode, epsilon, True)

    def __getitem__(self, i):
        """Return the element at position ``i``.

        ``self.__getitem__(i)`` <==> ``self[i]``

        Args:
            i (int or slice): index of the element

        Returns:
            element at position ``i``
        """
        if isinstance(i, slice):
            return SortedSet(self._impl.slice(i), self._typecode)
        return self._impl[i]

    def union(self, other):
        """Return a new ``SortedSet`` with the elements in one or both ``self``
        and ``other``.

        Values in ``other`` do not need to be in sorted order.

        ``self.union(other)`` <==> ``self | other``

        Args:
            other (iterable): a sequence of values

        Returns:
            SortedSet: new set with the elements in the union
        """
        args = SortedContainer._impl_or_iter(other)
        return SortedSet(self._impl.union(*args), self._typecode)

    __or__ = union

    def difference(self, other):
        """Return a ``SortedSet`` with the elements of ``self`` not found in
        ``other``.

        Values in ``other`` do not need to be in sorted order.

        ``self.difference(other)`` <==> ``self.__sub__(other)`` <==>
        ``self - other``

        Args:
            other (iterable): a sequence of values

        Returns:
            SortedSet: new set with the elements in the difference
        """
        args = SortedContainer._impl_or_iter(other)
        return SortedSet(self._impl.difference(*args), self._typecode)

    __sub__ = difference

    def symmetric_difference(self, other):
        """Return a ``SortedSet`` with the elements found in either ``self`` or
        ``other`` but not in both of them.

        Values in ``other`` do not need to be in sorted order.

        ``self.symmetric_difference(other)`` <==> ``self.__xor__(other)`` <==>
        ``self ^ other``

        Args:
            other (iterable): a sequence of values

        Returns:
            SortedSet: new set with the elements in the symmetric difference
        """
        args = SortedContainer._impl_or_iter(other)
        return SortedSet(self._impl.symmetric_difference(*args), self._typecode)

    __xor__ = symmetric_difference

    def intersection(self, other):
        """Return a ``SortedSet`` with the elements found in both ``self`` and
        ``other``.

        Values in ``other`` do not need to be in sorted order.

        ``self.intersection(other)`` <==> ``self.__and__(other)`` <==>
        ``self & other``

        Args:
            other (iterable): a sequence of values

        Returns:
            SortedSet: new set with the elements in the intersection
        """
        args = SortedContainer._impl_or_iter(other)
        return SortedSet(self._impl.intersection(*args), self._typecode)

    __and__ = intersection

    def copy(self):
        """Return a copy of ``self``.

        Returns:
            SortedSet: new set with the same elements of ``self``
        """
        return SortedSet(self._impl, self._typecode)

    __copy__ = copy

    def __eq__(self, other):
        """Return ``True`` if and only if ``self`` is equal to ``other``.

        ``self.__eq__(other)`` <==> ``self == other``

        Comparisons use `set <https://docs.python.org/3/library/stdtypes.html#set>`_
        semantics.

        Args:
            other (SortedSet or set): a sequence of values

        Returns:
            bool: ``True`` if sorted set is equal to ``other``
        """
        if isinstance(other, (SortedSet, set)):
            if len(self) != len(other):
                return False
            args = SortedContainer._impl_or_iter(other)
            return self._impl.equal_to(*args)
        return NotImplemented

    def __ne__(self, other):
        """Return ``True`` if and only if ``self`` is not equal to ``other``.

        ``self.__eq__(other)`` <==> ``self != other``

        Comparisons use `set <https://docs.python.org/3/library/stdtypes.html#set>`_
        semantics.

        Args:
            other (SortedSet or set): a sequence of values

        Returns:
            bool: ``True`` if sorted set is not equal to ``other``
        """
        if isinstance(other, (SortedSet, set)):
            if len(self) != len(other):
                return True
            args = SortedContainer._impl_or_iter(other)
            return self._impl.not_equal_to(*args)
        return NotImplemented

    def __lt__(self, other):
        """Return ``True`` if and only if ``self`` is a proper subset of ``other``.

        ``self.__lt__(other)`` <==> ``self < other``

        Comparisons use `set <https://docs.python.org/3/library/stdtypes.html#set>`_
        semantics.

        Args:
            other (SortedSet or set): a sequence of values

        Returns:
            bool: ``True`` if sorted set is a proper subset of ``other``
        """
        if isinstance(other, (SortedSet, set)):
            args = SortedContainer._impl_or_iter(other)
            return self._impl.subset(*args, True)
        return NotImplemented

    def __gt__(self, other):
        """Return ``True`` if and only if ``self`` is a proper superset of ``other``.

        ``self.__gt__(other)`` <==> ``self > other``

        Comparisons use `set <https://docs.python.org/3/library/stdtypes.html#set>`_
        semantics.

        Args:
            other (SortedSet or set): a sequence of values

        Returns:
            bool: ``True`` if sorted set is a proper superset of ``other``
        """
        if isinstance(other, (SortedSet, set)):
            args = SortedContainer._impl_or_iter(other)
            return self._impl.superset(*args, True)
        return NotImplemented

    def __le__(self, other):
        """Return ``True`` if and only if ``self`` is a subset of ``other``.

        ``self.__le__(other)`` <==> ``self <= other`` <==> 
        ``self.issubset(other)``

        Comparisons use `set <https://docs.python.org/3/library/stdtypes.html#set>`_
        semantics.

        Args:
            other (SortedSet or set): a sequence of values

        Returns:
            bool: ``True`` if sorted set is a subset of ``other``
        """
        if isinstance(other, (SortedSet, set)):
            args = SortedContainer._impl_or_iter(other)
            return self._impl.subset(*args, False)
        return NotImplemented

    def __ge__(self, other):
        """Return ``True`` if and only if ``self`` is a superset of ``other``.

        ``self.__ge__(other)`` <==> ``self >= other`` <==> 
        ``self.issuperset(other)``

        Comparisons use `set <https://docs.python.org/3/library/stdtypes.html#set>`_
        semantics.

        Args:
            other (SortedSet or set): a sequence of values

        Returns:
            bool: ``True`` if sorted set is a superset of ``other``
        """
        if isinstance(other, (SortedSet, set)):
            args = SortedContainer._impl_or_iter(other)
            return self._impl.superset(*args, False)
        return NotImplemented

    def isdisjoint(self, other):
        """Return ``True`` if and only if the set has no elements in common
        with ``other``.

        Sets are disjoint if and only if their intersection is the empty set.

        Args:
            other (iterable): a sequence of values

        Returns:
            bool: ``True`` if ``self`` is disjoint from ``other``
        """
        return len(self & other) == 0

    issubset = __le__
    issuperset = __ge__



__all__ = ['SortedList', 'SortedSet']
__version__ = '0.1'
__author__ = 'Giorgio Vinciguerra'

import os as _os

from .sortedlist import SortedList
from .sortedset import SortedSet

_os.environ['KMP_DUPLICATE_LIB_OK'] = 'True'



import glob
import os
import subprocess
import sys
import tempfile

import setuptools
from setuptools.command.build_ext import build_ext

__version__ = '0.1a2'


class get_pybind_include(object):
    """Helper class to determine the pybind11 include path."""

    def __str__(self):
        import pybind11
        return pybind11.get_include()


def has_flag(compiler, flag):
    """Check whether a flag is supported on the specified compiler."""
    with tempfile.NamedTemporaryFile('w', suffix='.cpp') as f:
        f.write('int main (int argc, char **argv) { return 0; }')
        try:
            compiler.compile([f.name], extra_postargs=[flag])
        except setuptools.distutils.errors.CompileError:
            return False
    return True


ext_modules = [
    setuptools.Extension(
        'pygm._pygm',
        ['pygm/pygm.cpp'],
        include_dirs=[
            get_pybind_include(),
            'PGM-index/include',
        ],
        language='c++'
    ),
]


def is_clang(bin):
    """Check whether the compiler is clang."""
    output = subprocess.check_output([bin, '-v'], stderr=subprocess.STDOUT)
    return 'clang' in output.decode('ascii', 'ignore')


class BuildExt(build_ext):
    """A custom build extension for adding compiler-specific options."""

    def build_extensions(self):
        comp_args = ['-std=c++17', '-O3', '-fvisibility=hidden']
        link_args = []

        if sys.platform == 'darwin' and is_clang(self.compiler.compiler[0]):
            omp = '/usr/local/opt/libomp'
            omp_lib = self.compiler.find_library_file([omp + '/lib'], 'omp')
            if omp_lib is not None:
                print('libomp found on macOS')
                comp_args += ['-Xpreprocessor',
                              '-fopenmp',
                              '-I%s/include/' % omp]
                link_args += ['-L%s/lib' % omp,
                              '-lomp']
            comp_args += ['-mmacosx-version-min=10.9']
            link_args += ['-mmacosx-version-min=10.9']
        elif has_flag(self.compiler, '-fopenmp'):
            comp_args += ['-fopenmp']
            link_args += ['-fopenmp']

        for ext in self.extensions:
            ext.extra_compile_args = comp_args
            ext.extra_link_args = link_args

        build_ext.build_extensions(self)


if sys.version_info[:2] < (3, 3):
    raise RuntimeError("Python version >= 3.3 required.")

if 'CXX' not in os.environ:  # try to use the latest gcc
    def gcc_sort_key(p):
        x = os.path.basename(p).split('-')[-1]
        return int(x) if x.isdigit() else 0

    path = os.getenv('PATH').split(os.path.pathsep)
    globs = [os.path.join(p, 'g++-*') for p in path]
    gccs = sorted([g for p in globs for g in glob.glob(p)], key=gcc_sort_key)
    if len(gccs) > 0:
        os.environ["CC"] = gccs[-1]
        os.environ["CXX"] = gccs[-1]
        print('Found GCC in', gccs[-1])

setuptools.setup(
    name='pygm',
    version=__version__,
    author='Giorgio Vinciguerra',
    author_email='i@gvdev.com',
    url='https://pgm.di.unipi.it/',
    project_urls={
        'Documentation': 'https://pgm.di.unipi.it/docs/python-reference/',
        'Source': 'https://github.com/gvinciguerra/PyGM/',
        'Tracker': 'https://github.com/gvinciguerra/PyGM/issues',
    },
    license='Apache-2.0',
    description=('Sorted containers with state-of-the-art query performance '
                 'and compressed memory usage'),
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    ext_modules=ext_modules,
    packages=setuptools.find_packages(),
    setup_requires=['pybind11>=2.5.0'],
    cmdclass={'build_ext': BuildExt},
    zip_safe=False,
    classifiers=[
        'License :: OSI Approved :: Apache Software License',
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'Intended Audience :: Science/Research',
        'Natural Language :: English',
        'Programming Language :: C++',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Topic :: Database',
        'Topic :: Scientific/Engineering',
        'Topic :: Scientific/Engineering :: Artificial Intelligence',
        'Topic :: Scientific/Engineering :: Bio-Informatics',
        'Topic :: Scientific/Engineering :: Information Analysis',
        'Topic :: Software Development',
        'Topic :: System',
        'Topic :: System :: Archiving :: Compression',
        'Topic :: Utilities',
    ],
    keywords=('tree list array btree b+tree vector skiplist container '
              'sortedlist sorted set query index data structure'),
)
